@@ -8,7 +8,7 @@ import logging
 log = logging.getLogger("modules.Flickr")
 
 import conduit
-import conduit.Utils as Utils
+import conduit.utils as Utils
 import conduit.Web as Web
 import conduit.dataproviders.Image as Image
 import conduit.Exceptions as Exceptions
@@ -21,13 +21,16 @@ from gettext import gettext as _
 #http://sourceforge.net/tracker/index.php?func=detail&aid=1874067&group_id=203043&atid=984009
 Utils.dataprovider_add_dir_to_path(__file__)
 import flickrapi
-if not flickrapi.__version__.endswith("CONDUIT"):
-    raise Exceptions.NotSupportedError("Incorrect Flickrapi version")
-
-MODULES = {
-	"FlickrTwoWay" :          { "type": "dataprovider" }        
-}
 
+if flickrapi.__version__.endswith("CONDUIT"):
+    MODULES = {
+    	"FlickrTwoWay" :          { "type": "dataprovider" }        
+    }
+    log.info("Module Information: %s" % Utils.get_module_information(flickrapi, "__version__"))
+else:
+    MODULES = {}
+    log.info("Flickr support disabled")
+    
 class MyFlickrAPI(flickrapi.FlickrAPI):
     def __init__(self, apiKey, secret, username):
             flickrapi.FlickrAPI.__init__(self, 
@@ -73,6 +76,7 @@ class FlickrTwoWay(Image.ImageTwoWay):
         self.fapi = None
         self.token = None
         self.username = ""
+        self.logged_in = False
         self.photoSetName = ""
         self.showPublic = True
         self.photoSetId = None
@@ -121,75 +125,92 @@ class FlickrTwoWay(Image.ImageTwoWay):
         # get the id
         photoId = ret.photoid[0].elementText
 
-        # check if phtotoset exists, create it otherwise add photo to it
+        # check if phtotoset exists, if not create it
         if not self.photoSetId:
-            # create one with created photoID if not
-            ret = self.fapi.photosets_create(
-                                        title=self.photoSetName,
-                                        primary_photo_id=photoId)
-
-            if self.fapi.getRspErrorCode(ret) != 0:
-                raise Exceptions.SyncronizeError("Flickr failed to create photoset: %s" % self.fapi.getPrintableError(ret))
-
-            self.photoSetId = ret.photoset[0]['id']
-        else:
-            # add photo to photoset
+            self.photoSetId = self._create_photoset(photoId)
+        # add the photo to the photoset
+        if self.photoSetId:
             ret = self.fapi.photosets_addPhoto(
-                                            photoset_id = self.photoSetId,
-                                            photo_id = photoId)
-
+                                photoset_id = self.photoSetId,
+                                photo_id = photoId)
             if self.fapi.getRspErrorCode(ret) != 0:
-                raise Exceptions.SyncronizeError("Flickr failed to add photo to set: %s" % self.fapi.getPrintableError(ret))
+                log.warn("Flickr failed to add photo to set: %s" % self.fapi.getPrintableError(ret))
 
         #return the photoID
         return Rid(uid=photoId)
 
-    
     def _get_photo_size (self):
         return self.imageSize
+
+    def _set_username(self, username):
+        if self.username != username:
+            self.username = username
+            self.logged_in = False        
+
+    def _login(self):
+        #only log in if we need to
+        if not self.logged_in:
+            self.fapi = MyFlickrAPI(FlickrTwoWay.API_KEY, FlickrTwoWay.SHARED_SECRET, self.username)
+            self.token = self.fapi.login()
+            self.logged_in = True
+            
+    def _create_photoset(self, primaryPhotoId):
+        #create one with created photoID if not
+        ret = self.fapi.photosets_create(
+                                title=self.photoSetName,
+                                primary_photo_id=primaryPhotoId)
+
+        photoSetId = None
+        if self.fapi.getRspErrorCode(ret) != 0:
+            log.warn("Flickr failed to create photoset: %s" % self.fapi.getPrintableError(ret))
+        else:
+            photoSetId = ret.photoset[0]['id']
+        return photoSetId
+                
+    def _get_photoset(self):
+        for name, photoSetId in self._get_photosets():
+            if name == self.photoSetName:
+                log.debug("Found album %s" % self.photoSetName)
+                self.photoSetId = photoSetId
+                
+    def _get_photosets(self):
+        ret = self.fapi.photosets_getList()  
+        if self.fapi.getRspErrorCode(ret) != 0:
+            log.warn("Flickr Refresh Error: %s" % self.fapi.getPrintableError(ret))
+            return []
+
+        photosets = []
+        if hasattr(ret.photosets[0], 'photoset'):
+            for pset in ret.photosets[0].photoset:
+                photosets.append(
+                            (pset.title[0].elementText, #photoset name
+                            pset['id']))                #photoset id
+
+        return photosets
+        
+    def _get_photos(self):
+        if not self.photoSetId:
+            return []
+
+        photoList = []
+        ret = self.fapi.photosets_getPhotos(photoset_id=self.photoSetId)
+        if self.fapi.getRspErrorCode(ret) != 0:
+            log.warn("Flickr failed to get photos: %s" % self.fapi.getPrintableError(ret))
+        else:            
+            for photo in ret.photoset[0].photo:
+                photoList.append(photo['id'])
+        return photoList
         
     # DataProvider methods
     def refresh(self):
         Image.ImageTwoWay.refresh(self)
-        #only log in if we need to
-        if self.fapi == None:
-            self.fapi = MyFlickrAPI(FlickrTwoWay.API_KEY, FlickrTwoWay.SHARED_SECRET, self.username)
-            self.token = self.fapi.login()
-            
-        # try to get the photoSetId
-        ret = self.fapi.photosets_getList()
-
-        if self.fapi.getRspErrorCode(ret) != 0:
-            raise Exceptions.RefreshError("Flickr Refresh Error: %s" % self.fapi.getPrintableError(ret))
-
-        if not hasattr(ret.photosets[0], 'photoset'):
-            return
-
-        # look up the photo set
-        for set in ret.photosets[0].photoset:
-            if set.title[0].elementText == self.photoSetName:
-                self.photoSetId = set['id']      
-
+        self._login()
+        self._get_photoset()            
         used,tot,percent = self._get_user_quota()
         log.debug("Used %2.1f%% of monthly badwidth quota (%skb/%skb)" % (percent,used,tot))
 
     def get_all(self):
-        # return  photos list is filled, raise error if not
-        if not self.photoSetId:
-            return []
-
-        ret = self.fapi.photosets_getPhotos(photoset_id=self.photoSetId)
-
-        if self.fapi.getRspErrorCode (ret) != 0:
-            raise Exceptions.SyncronizeError("Flickr failed to get photos: %s" % self.fapi.getPrintableError(ret))
-
-        photoList = []
-
-        for photo in ret.photoset[0].photo:
-            photoList.append(photo['id'])
-
-        return photoList
-
+        return self._get_photos()
 
     def get (self, LUID):
         # get photo info
@@ -200,7 +221,9 @@ class FlickrTwoWay(Image.ImageTwoWay):
         title = str(photoInfo.photo[0].title[0].elementText)
         # get tags
         tagsNode = photoInfo.photo[0].tags[0]
-
+        # get caption
+        caption = photoInfo.photo[0].description[0].elementText
+        
         if hasattr(tagsNode, 'tag'):
             tags = tuple(tag.elementText for tag in tagsNode.tag)
         else:
@@ -209,6 +232,7 @@ class FlickrTwoWay(Image.ImageTwoWay):
         # create the file
         f = Photo.Photo (URI=url)
         f.set_open_URI(url)
+        f.set_caption(caption)
 
         # try to rename if a title is available
         # FIXME: this is far from optimal, also there should be 
@@ -239,37 +263,87 @@ class FlickrTwoWay(Image.ImageTwoWay):
         """
         Configures the Flickr sink
         """
+        import gobject
+        import gtk
+        def on_login_finish(*args):
+            if self.logged_in:
+                build_photoset_model()
+            Utils.dialog_reset_cursor(dlg)
+                
+        def on_response(sender, responseID):
+            if responseID == gtk.RESPONSE_OK:
+                self._set_username(username.get_text())
+                self.photoSetName = photosetCombo.get_active_text()
+                self.showPublic = publicCb.get_active()
+                self.imageSize = self._resize_combobox_get_active(resizecombobox)
+        
+        def load_click(button, window, usernameEntry):
+            self._set_username(usernameEntry.get_text())
+            Utils.dialog_set_busy_cursor(dlg)
+            conduit.GLOBALS.syncManager.run_blocking_dataprovider_function_calls(
+                                            self,
+                                            on_login_finish,
+                                            self._login)            
+
+        def username_changed(entry, load_button):
+            load_button.set_sensitive (len(entry.get_text()) > 0)
+
+        def build_photoset_model():
+            photoset_store.clear()
+            photoset_count = 0
+            photoset_iter = None
+            for name, photoSetId in self._get_photosets():       
+                iter = photoset_store.append((name,))
+                if name == self.photoSetName:
+                    photoset_iter = iter
+                photoset_count += 1
+
+            if photoset_iter:
+                photosetCombo.set_active_iter(photoset_iter)
+            elif self.photoSetName:
+                photosetCombo.child.set_text(self.photoSetName)
+            elif photoset_count:
+                photosetCombo.set_active(0)
+ 
+        #get a whole bunch of widgets
         tree = Utils.dataprovider_glade_get_widget(
                         __file__, 
                         "config.glade", 
                         "FlickrTwoWayConfigDialog")
-        
-        #get a whole bunch of widgets
-        photoSetEntry = tree.get_widget("photoset_entry")
+        photosetCombo = tree.get_widget("photoset_combo")
         publicCb = tree.get_widget("public_check")
         username = tree.get_widget("username")
+        load_button = tree.get_widget('load_button')
+        ok_button = tree.get_widget('ok_button')
+        dlg = tree.get_widget("FlickrTwoWayConfigDialog")
 
         resizecombobox = tree.get_widget("resizecombobox")
         self._resize_combobox_build(resizecombobox, self.imageSize)
+
+        #signals
+        load_button.connect('clicked', load_click, window, username)
+        username.connect('changed', username_changed, load_button)
         
         #preload the widgets
-        photoSetEntry.set_text(self.photoSetName)
         publicCb.set_active(self.showPublic)
         username.set_text(self.username)
-        
-        dlg = tree.get_widget("FlickrTwoWayConfigDialog")
-        response = Utils.run_dialog(dlg, window)
 
-        if response == True:
-            # get the values from the widgets
-            self.photoSetName = photoSetEntry.get_text()
-            self.showPublic = publicCb.get_active()
-            self.username = username.get_text()
-            self.imageSize = self._resize_combobox_get_active(resizecombobox)
-        dlg.destroy()    
+        #setup photoset combo
+        photoset_store = gtk.ListStore (gobject.TYPE_STRING)
+        photosetCombo.set_model(photoset_store)
+        cell = gtk.CellRendererText()
+        photosetCombo.pack_start(cell, True)
+        photosetCombo.set_text_column(0)
+
+        #disable photoset lookup if no username entered
+        enabled = len(self.username) > 0
+        load_button.set_sensitive(enabled)
+        
+        # run dialog 
+        Utils.run_dialog_non_blocking(dlg, on_response, window)
        
     def is_configured (self, isSource, isTwoWay):
-        return len (self.username) > 0
+        return len(self.username) > 0 and len(self.photoSetName) > 0
         
     def get_configuration(self):
         return {
